//
//  DSListViewCell.m
//  WohnungsAppnahme
//
//  Created by Boris Bügling on 02.08.13.
//  Copyright (c) 2013 DAVID Software GmbH. All rights reserved.
//

#import <JRSwizzle/JRSwizzle.h>
#import <objc/runtime.h>
#import <QuartzCore/QuartzCore.h>

#import "DSListViewCell.h"

@interface UIView (Private)

@property(nonatomic, getter=isRotated) BOOL rotated;

@end

#pragma mark -

@interface DSListViewCell ()

@property (weak) UIButton* deleteConfirmButton;
@property (weak) UIControl* editControl;
@property (weak) UIImageView* minusButton;

@end

#pragma mark -

@implementation DSListViewCell

+(void)load {
    Class class = NSClassFromString(@"UITableViewCellEditControl");
    SEL originalSelector = NSSelectorFromString(@"_toggleRotateAnimationDidStop:finished:");
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wundeclared-selector"
    SEL newSelector = @selector(toggleMinusButtonAnimationDidStop:finished:);
#pragma clang diagnostic pop

    IMP imp = imp_implementationWithBlock(^(id target){
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
        [target performSelector:newSelector];
#pragma clang diagnostic pop

        [self toggleMinusButtonImageForControlView:target];
    });

    Method m = class_getInstanceMethod(class, originalSelector);
    if (!class_addMethod(class, newSelector, imp, method_getTypeEncoding(m))) {
        return;
    }

    NSError* error;
    if (![class jr_swizzleMethod:originalSelector withMethod:newSelector error:&error]) {
        NSLog(@"Error swizzling finished callback: %@", error);
    }
}

+(UIImageView*)toggleMinusButtonImageForControlView:(UIView*)controlView {
    if (controlView.subviews.count != 1 || ![controlView.subviews[0] isKindOfClass:[UIImageView class]]) {
        return nil;
    }

    UIImageView* imageView = controlView.subviews[0];
    imageView.image = [UIImage imageNamed:(controlView.rotated ? @"minus-empty" : @"minus")];
    return imageView;
}

#pragma mark -

-(BOOL)canBecomeFirstResponder {
    return YES;
}

-(BOOL)canPerformAction:(SEL)action withSender:(id)sender {
    return [NSStringFromSelector(action) isEqualToString:@"copy:"];
}

-(void)copy:(id)sender {
    [self.delegate copySelectedOnCell:self];
}

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
    self = [super initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:reuseIdentifier];
    return self;
}

-(void)willTransitionToState:(UITableViewCellStateMask)state {
    [super willTransitionToState:state];

    if (![self state:UITableViewCellStateShowingEditControlMask isInCurrentStateMask:state]) {
        [self.minusButton removeObserver:self forKeyPath:@"image" context:NULL];

        self.editControl = nil;
        self.minusButton = nil;
    }

    for (UIView* subview in self.subviews) {
        if ([self state:UITableViewCellStateShowingDeleteConfirmationMask isInCurrentStateMask:state]) {
            [self replaceDeleteConfirmationButtonInSubView:subview];
        }

        if ([self state:UITableViewCellStateShowingEditControlMask isInCurrentStateMask:state]) {
            [self replaceEditControlInSubview:subview];
        }
    }
}

#pragma mark -

- (void)dealloc {
    if (self.minusButton) {
        [self.minusButton removeObserver:self forKeyPath:@"image" context:NULL];
    }
    self.minusButton = nil;
}

-(void)editControlTapped {
    self.minusButton.image = [UIImage imageNamed:@"minus-empty"];
}

-(void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change
                      context:(void *)context {
    UIImage* newImage = change[NSKeyValueChangeNewKey];
    UIImage* oldImage = change[NSKeyValueChangeOldKey];

    if (newImage == oldImage) {
        return;
    }

    NSArray* callStack = [NSThread callStackSymbols];
    if (callStack.count > 5) {
        if ([callStack[5] rangeOfString:@"updateImageView"].location != NSNotFound) {
            [object setValue:oldImage forKeyPath:keyPath];
        }
    }
}

-(void)replaceDeleteConfirmationButtonInSubView:(UIView*)subview {
    if ([NSStringFromClass([subview class]) rangeOfString:@"DeleteConfirm"].location == NSNotFound) {
        return;
    }

    if (subview.subviews.count > 1) {
        return;
    }

    UILabel* label = [[UILabel alloc] initWithFrame:subview.bounds];
    label.backgroundColor = [UIColor colorWithRed:37.0/255.0 green:50.0/255.0 blue:134.0/255.0 alpha:1.0];
    label.height -= 2.0;
    label.text = NSLocalizedString(@"Delete", nil);
    label.textAlignment = NSTextAlignmentCenter;
    label.textColor = [UIColor whiteColor];
    label.y = 1.0;
    [subview addSubview:label];
}

-(void)replaceEditControlInSubview:(UIView*)subview {
    if (self.editControl) {
        self.minusButton.image = [UIImage imageNamed:@"minus-empty"];
        return;
    }

    if ([NSStringFromClass([subview class]) rangeOfString:@"CellEdit"].location == NSNotFound) {
        return;
    }

    self.minusButton = [[self class] toggleMinusButtonImageForControlView:subview];
    [self.minusButton addObserver:self
                       forKeyPath:@"image"
                          options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld
                          context:nil];
    
    if (!self.minusButton) {
        return;
    }

    self.editControl = (UIControl*)subview;
    [self.editControl addTarget:self
                         action:@selector(editControlTapped)
               forControlEvents:UIControlEventTouchUpInside];
}

-(BOOL)state:(UITableViewCellStateMask)state isInCurrentStateMask:(UITableViewCellStateMask)mask {
    return (mask & state) == state;
}

@end
